//---------------------------------------------------------------------------

#include <windows.h>
#include <stdio.h>
#include <conio.h>

//Includes fr PKCS#11
#include "cryptoki.h"

#pragma hdrstop

//---------------------------------------------------------------------------

#pragma argsused
void printBytes(CK_BYTE_PTR pBytes, DWORD dLenBytes) {

	for (DWORD d = 0; d < dLenBytes; d++) {
		printf("0x%X ",(pBytes[d]));
	}
}

int main(int argc, char* argv[])
{

   unsigned char ucPWDUser[] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36 };
   DWORD dwLenPWDUser = 6;

   CK_FUNCTION_LIST_PTR p11;
   CK_RV returnCode;
   CK_C_GetFunctionList C_GetFunctionListDyn;

   if (argc != 2) {
		printf("Bitte LibName als Argument angeben");
		exit(0);
   }
   //DLL laden
   HINSTANCE hDll = LoadLibrary(argv[1]);
   if (hDll == NULL) {
		printf("Fehler beim Laden der Bibliothek");
		exit(0);
   }

   //Funktionen holen
   C_GetFunctionListDyn = (CK_C_GetFunctionList) GetProcAddress(hDll, "C_GetFunctionList");

   //Liste der Funktionen holen
   returnCode = C_GetFunctionListDyn(&p11);
   if (returnCode != CKR_OK) {
		FreeLibrary(hDll);
		hDll = NULL;
		p11 = NULL;
		printf("\nFehler bei C_GetFunctionList (dyn)");
		exit(0);
   }

   //PKCS#11 initialisieren
   returnCode = p11->C_Initialize(NULL_PTR);
   if (returnCode != CKR_OK) {
		printf("\nFehler bei C_Initialize");
		getch();
		exit(0);
   } else {
		printf("\nC_Initialize erfolgreich");
   }

   //Anzahl Slots holen
   CK_ULONG ulSlotCount;
   returnCode = p11->C_GetSlotList(CK_TRUE, NULL_PTR, &ulSlotCount);
   if (returnCode != CKR_OK) {
		printf("\nFehler bei C_GetSlotList");
		getch();
		exit(0);
   } else {
		printf("\nC_GetSlotList erfolgreich");
   }

   //Sind Slots da?
   if (ulSlotCount == 0) {
		printf("\nKeine Slots gefunden");
		getch();
		exit(0);
   } else {
		printf("\n %d Slots gefunden !",ulSlotCount);
   }

   //Slots holen
   CK_SLOT_ID_PTR pSlotId = (CK_SLOT_ID_PTR) malloc(sizeof(CK_SLOT_ID) * ulSlotCount);
   returnCode = p11->C_GetSlotList(CK_TRUE, pSlotId, &ulSlotCount);
   if (returnCode != CKR_OK) {
		printf("\nFehler bei C_GetSlotList");
		getch();
		exit(0);
   } else {
		printf("\nC_GetSlotList erfolgreich, verwende SlotID : %X",pSlotId[0]);
   }

   //1. Slot verwenden !!
   //Session ffnen
   CK_SESSION_HANDLE hSession;
   returnCode = p11->C_OpenSession(pSlotId[0], CKF_SERIAL_SESSION, "Test-InitPin", NULL_PTR, &hSession);
   //War das OK?
   if (returnCode != CKR_OK) {
		printf("\nFehler bei C_OpenSession");
		getch();
		exit(0);
   } else {
		printf("\nC_OpenSession erfolgreich, Session Handle : %X",hSession);
   }

   // Auf PinInitialisierung prfen
   //Infos zum Token holen
   CK_TOKEN_INFO tokenInfo;
   returnCode = p11->C_GetTokenInfo(pSlotId[0], &tokenInfo);
   if (returnCode != CKR_OK) {
		printf("\nFehler bei C_GetTokenInfo");
		getch();
		exit(0);
   }

   if ( (tokenInfo.flags & CKF_USER_PIN_INITIALIZED) != 0x00) {
		printf("\nPin dieses Tokens bereits initialisiert !");
		getch();
		exit(0);
   }

   //Pin initialisieren
   returnCode = p11->C_InitPIN(hSession, "123456", 6);
   if (returnCode != CKR_OK)  {
		printf("\nFehler bei C_InitPIN");
		getch();
		exit(0);
   }

   // Alternativ ebenfalls verwendbar: Pin ndern
   unsigned char ucInitPin[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

/*
   //Pin ndern
   returnCode = p11->C_SetPIN(hSession, ucInitPin, 6, "123456", 6);
   if (returnCode != CKR_OK)  {
		printf("\nFehler bei C_SetPin");
		getch();
		exit(0);
   }
*/

   //Warten
   printf("\nTest erfolgreich !");
   getch();

}
//---------------------------------------------------------------------------
